@@ -24,8 +24,66 @@
 
 #include "env.c"
 
+
+#if defined __linux__ || defined __FreeBSD__
+
+/*  Look at /proc/$$/{exe,file} for the current executable 
+
+    Returns malloc()ed string.  Caller must free.
+    Returns NULL if can't be found.
+    
+    Note that FreeBSD has /proc unmounted by default.  You'd think we could
+    get this info via the kvm interface, but it turns out that to get
+    kvm_getprocs()/kvm_read() to return any information we don't already
+    have, we need read-access to /boot/kmem, which we don't have.  And I
+    couldn't get to work anyway.  Email me (philip-at-pied.nu) if want a
+    stab at the code. */
+
+char *par_current_exec_proc( void ) 
+{
+    char proc_path[MAXPATHLEN + 1], link[MAXPATHLEN + 1];
+    char *ret = NULL;
+    int n;
+    
+    n = sprintf( proc_path, "/proc/%i/%s", (int)getpid(), 
+#if defined __FreeBSD__
+        "file"
+#else
+        "exe"
+#endif
+    );
+    if( n < 0 ) 
+        return NULL;
+
+    n = readlink( proc_path, link, MAXPATHLEN);
+    if( n < 0 )
+        return NULL;
+    
+    ret = (char *)malloc( n+1 );
+    if( ret == NULL )
+        return NULL;
+
+    memcpy( ret, link, n );
+    ret[n] = '\0';
+
+    return ret;
+} 
+
+#endif
+
+char *par_current_exec( void )
+{
+#if defined __linux__ || defined __FreeBSD__
+    return par_current_exec_proc();
+#else
+    return NULL;
+#endif
+}
+
+
+
 char *par_findprog(char *prog, char *path) {
-    char *p, filename[MAXPATHLEN];
+    char *p, *ret, filename[MAXPATHLEN];
     int proglen, plen;
     char *par_temp = par_getenv("PAR_TEMP");
 
@@ -46,6 +104,22 @@ struct stat PL_statbuf;
         return(prog);
     }
 
+#if defined __linux__ || defined __FreeBSD__
+    ret = par_current_exec_proc();
+#else
+    ret = NULL;
+#endif
+
+    if( ret != NULL ) {
+        par_setenv( "PAR_PROGNAME", ret );
+        return ret;
+    }
+
+    /* Walk through PATH (path), looking for ourself (prog).
+        This fails if we are invoked in an obscure manner;
+        Basically, execvp( "/full/path/to/prog", "prog", NULL ) and
+        "/full/path/to" isn't in $PATH.  Of course, I can't think 
+        of a situation this will happen. */
     proglen = strlen(prog);
     p = strtok(path, path_sep);
     while ( p != NULL ) {
@@ -81,6 +155,7 @@ struct stat PL_statbuf;
     return(prog);
 }
 
+
 char *par_basename (const char *name) {
     const char *base = name;
     const char *p;
@@ -92,6 +167,9 @@ char *par_basename (const char *name) {
     return (char *)base;
 }
 
+
+
+
 char *par_dirname (const char *path) {
     static char bname[MAXPATHLEN];
     register const char *endp;
@@ -148,6 +226,10 @@ void par_init_env () {
         par_setenv("PAR_DEBUG", buf);
     }
 
+    if ( (buf = par_getenv("PAR_GLOBAL_TMPDIR")) != NULL ) {
+        par_setenv("PAR_TMPDIR", buf);
+    }
+
     if ( (buf = par_getenv("PAR_GLOBAL_TEMP")) != NULL ) {
         par_setenv("PAR_TEMP", buf);
     }
